#!/usr/bin/env python3
# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-

#
# Ubuntu Sanity Tests
# Copyright (C) 2014, 2015 Canonical
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

import logging
import os
import subprocess
import sys

from argparse import ArgumentParser
from ubuntu_sanity_tests import (
    config,
    results
)

logger = logging.getLogger(__name__)
_ADT_RUN_SHORT_TIMEOUT = 600  # 10 minutes.


def main(argv=None):
    if argv is None:
        argv = sys.argv[1:]
    args = _parse_arguments(argv)
    return run_sanity_tests(args.config, args.suites,
                            args.update_apt, args.network)


def _parse_arguments(argv=None):
    parser = ArgumentParser('Run sanity tests for Ubuntu Touch.')
    parser.add_argument('--config', '-c',
                        help='The config file to use for this run.',
                        default='ubuntu-sanity-tests.conf')
    parser.add_argument(
        '--update-apt', '-u', action='store_true',
        help='Update apt lists before installing the dependencies',
        default=False)
    parser.add_argument(
        '--network', '-n', action='store_true',
        help='Setup the network using phablet-network')
    parser.add_argument(
        'suites', nargs='*', help='The list of test suites to run.')
    return parser.parse_args(args=argv)


def run_sanity_tests(conf_name, tests_to_run, update_apt=False, network=False):
    """Run sanity tests in a device connected via USB and report the results.

    :param conf_name: The name of the configuration file that will be used
        for the sanity tests.
    :param tests_to_run: A list of tests to run.
    :param update_apt: Boolean representing whether apt-get update should be
        run on the device before installing dependencies.
    :param network: Boolean representing whether phablet-network should be run
        to provision the network on the device.
    :returns: 0 when successful, other value when failed.

    """
    config_stack = _get_config_stack(conf_name, tests_to_run)
    print_camera_warning()

    if network:
        setup_network(config_stack)
    enable_autopilot_introspection(config_stack)
    return_code = run_autopkg_test(config_stack, update_apt)

    if return_code <= 6:
        # Return codes higher than 6 mean that there was a problem with adt or
        # the test bed, not a failure in the tests. So there will be no file
        # with results.
        generate_user_report(config_stack)

    return return_code


def print_camera_warning():
    """Print a warning to raise device when running camera test."""
    msg = (' Please ensure device is raised from surface to allow rear camera '
           'to take a photograph during camera tests.')
    msg_fmt = ('\n{breakline}\n{message}\n{breakline}\n')
    logger.warn(msg_fmt.format(breakline='*' * len(msg), message=msg))


def _get_config_stack(conf_name, tests_to_run):
    """Get the configuration values for the sanity tests.

    This function registers the declared options, opens the config store
    using the specified file name, adds the options coming from the command
    line, and asks the user for the missing options.

    """
    config.register_test_options()

    config_stack = config.UbuntuSanityTestsStack(conf_name)

    # We set the values passed from the command line to send them from the host
    # to the testbed.
    config_stack.set('tests_to_run', ' '.join(tests_to_run or []))

    config_stack.get_missing_options_from_stdin()

    return config_stack


def setup_network(config_stack):
    command = _build_setup_network_command(config_stack)
    subprocess.check_output(command)


def _build_setup_network_command(config_stack):
    command = ['phablet-network'] + _get_serial_argument(config_stack)
    return command


def _get_serial_argument(config_stack):
    device_serial = config_stack.get('device_serial')
    if device_serial:
        return ['-s', device_serial]
    else:
        # No serial in the config, run the commands as if only one device is
        # connected.
        return []


def enable_autopilot_introspection(config_stack):
    command = _build_enable_autopilot_introspection_command(config_stack)
    subprocess.check_output(command)


def _build_enable_autopilot_introspection_command(config_stack):
    command = (
        ['phablet-config'] +
        _get_serial_argument(config_stack) +
        ['autopilot', '--dbus-probe', 'enable']
    )
    return command


def run_autopkg_test(config_stack, update_apt):
    adt_run_cmd = _build_adt_run_test_command(config_stack, update_apt)
    return subprocess.call(adt_run_cmd)


def _build_adt_run_test_command(config_stack, update_apt=False):
    output_dir = config_stack.get('output_dir')
    device_password = config_stack.get('device_password')

    run_cmd = [
        'adt-run',
        # Updating and installing can take a while.
        '--timeout-short={}'.format(_ADT_RUN_SHORT_TIMEOUT),
        '-B',
        '--unbuilt-tree={}'.format(os.getcwd()),
        '--output-dir',
        output_dir,
        # Even if the config file on the host has a different name, on the
        # testbed it's copied with the default name.
        '--copy',
        '{}:/home/phablet/.config/ubuntu-sanity-tests.conf'.format(
            config_stack.store.path),
    ]

    ppa_setup_command = _get_ppa_setup_command()

    if update_apt:
        run_cmd.extend(['--setup-commands', 'ro-apt-update'])

    ssh_cmd = (
        ['ssh', '-s', 'adb', '--', '-p', device_password] +
        _get_serial_argument(config_stack)
    )
    return run_cmd + ppa_setup_command + ['---'] + ssh_cmd


def _get_ppa_setup_command():
    return [
        '--setup-commands',
        'mount -o remount,rw /',
        '--setup-commands',
        'apt-add-repository -y ppa:canonical-platform-qa/selenium',
        '--setup-commands',
        'apt-get --no-list-cleanup update -o Dir::Etc::SourceList=/dev/null',
        '--setup-commands',
        'sync; sleep 2; mount -o remount,ro /'
    ]


def generate_user_report(config_stack):
    """Create a human readable report from the subunit results."""
    artifacts_directory = results.get_artifacts_directory(config_stack)
    results.convert_subunit_results_to_pyunit(artifacts_directory)
    results.print_results(artifacts_directory)


if __name__ == '__main__':
    sys.exit(main())
