# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-

#
# Ubuntu Sanity Tests
# Copyright (C) 2014, 2015 Canonical
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

import os

from uciconfig import (
    options,
    stacks,
    stores
)

UbuntuSanityTestsStore = stores.FileStore


CONFIG_OPTIONS = [
    options.Option(
        'device_serial', help_string='Device serial'),
    options.Option(
        'device_password', default=options.MANDATORY,
        help_string='Device password'),
    options.Option(
        'output_dir', default=options.MANDATORY,
        help_string='Directory to store the tests results and artifacts'),
    options.Option(
        'sim_0_pin', default=options.MANDATORY,
        help_string='SIM pin for the card in the first slot'),
    options.Option(
        'wifi_ssid', default=options.MANDATORY,
        help_string='Wi-Fi SSID used to connect during setup wizard'),
    options.Option(
        'wifi_password', default=options.MANDATORY,
        help_string='Wi-Fi password used to connect during setup wizard'),
    options.Option(
        'language', default='English (United States)',
        help_string='The language of device to select during setup wizard'),
    options.Option(
        'device_security', default='Passcode',
        help_string='The security method to select during setup wizard'),
    options.Option(
        'bluetooth_device_name', 
        help_string='Name of a nearby bluetooth pairing device'),
]
"""Options for tests in external debian and click packages."""


def get_user_config_dir():
    """Return the path to the user configuration directory."""
    conf_dir = os.environ.get(
        'XDG_CONFIG_HOME', os.path.expanduser('~/.config'))
    return conf_dir


def register_test_options():
    """Register the options for the Ubuntu Sanity Tests."""
    for option in CONFIG_OPTIONS:
        options.option_registry.register(option)


class UbuntuSanityTestsStack(stacks.Stack):

    def __init__(self, conf_name='ubuntu-sanity-tests.conf'):
        config_path = os.path.join(
            get_user_config_dir(), conf_name)
        user_store = self.get_shared_store(UbuntuSanityTestsStore(config_path))
        super().__init__(
            [stacks.NameMatcher(user_store, None).get_sections],
            user_store, mutable_section_id=None)

    def get_missing_options_from_stdin(self):
        """Ask the user for the values of the missing options.

        The values entered by the user will be saved in the config file.

        """
        user_config_dir = get_user_config_dir()
        if not os.path.exists(user_config_dir):
            os.mkdir(user_config_dir)

        for key in options.option_registry.keys():
            if not self._is_option_in_store(key):
                self._get_option_value_from_stdin(key)

        self.store.save()

    def _is_option_in_store(self, key):
        for store, section in self.iter_sections():
            if section.get(key, None) is not None:
                return True
        else:
            return False

    def _get_option_value_from_stdin(self, key):
        help_ = self._get_option_help_text(key)
        # This will give a pyflakes error if it is installed in py2
        # See http://pad.lv/1331267
        print(help_, end='')  # flake8: noqa
        value = self._get_value_from_user_or_default(key)
        self.set(key, value)

    def _get_option_help_text(self, key):
        help_ = options.option_registry.get_help(key)
        default = options.option_registry.get(key).default
        if default is not options.MANDATORY:
            if default:
                help_ += ' (leave blank to use default value: {})'.format(
                    default)
            else:
                help_ += ' (leave blank if not required)'
        help_ += ': '
        return help_

    def _get_value_from_user_or_default(self, key):
        value = input()
        if not value and self.get(key, None):
            # If the option was left blank, check if it has a default and
            # use it instead.
            value = self.get(key)
        return value

    def get_autopilot_config_string(self):
        """Return the configuration in a string usable by Autopilot.

        That is, a list of key=value pairs serparated by comma.

        """
        self.get_missing_options_from_stdin()
        return ','.join(
            [key+'='+value for _, _, key, value in self.iter_options()])
