# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-

#
# Ubuntu Sanity Tests
# Copyright (C) 2014 Canonical
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

import dbus
import logging

logger = logging.getLogger(__name__)


def _get_sim_manager(sim_number):
    bus = dbus.SystemBus()
    modem = bus.get_object('org.ofono', '/ril_{}'.format(sim_number))
    return dbus.Interface(modem, 'org.ofono.SimManager')


def is_pin_locked(sim_number):
    sim_manager = _get_sim_manager(sim_number)
    locked_pins = sim_manager.GetProperties()['LockedPins']
    return 'pin' in locked_pins


def unlock_sim(sim_number, pin):
    sim_manager = _get_sim_manager(sim_number)
    if is_pin_locked(sim_number):
        sim_manager.UnlockPin('pin', pin)


def lock_sim(sim_number, pin):
    sim_manager = _get_sim_manager(sim_number)
    if not is_pin_locked(sim_number):
        sim_manager.LockPin('pin', pin)


def is_sim_inserted(sim_number=None):
    """Check a SIM is inserted.

    Checks if a SIM card is inserted in the specified
    slot on the device. If no slot is specified then
    all slots will be checked

    :param sim_number: The slot index to check for SIM presence

    :returns: True if a SIM is inserted in the specified slot or
    if no slot is specified then if a SIM is inserted in
    any slot, otherwise False
    """
    bus = dbus.SystemBus()
    manager = dbus.Interface(
        bus.get_object('org.ofono', '/'), 'org.ofono.Manager')
    modems = manager.GetModems()
    for ril_modem, properties in modems:
        # Only check the presence of the SIM if it's the
        # specified SIM or if no SIM number was specified
        if str(ril_modem).endswith(str(sim_number)) or sim_number is None:
            modem = bus.get_object('org.ofono', ril_modem)
            sim_manager = dbus.Interface(modem, 'org.ofono.SimManager')
            try:
                if sim_manager.GetProperties()['Present']:
                    return True
            except dbus.exceptions.DBusException as e:
                logger.warning('exception: {0}detected on modem: {1}'.format(
                    e, ril_modem))
                continue
    return False
