# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-

#
# Ubuntu Sanity Tests
# Copyright (C) 2015 Canonical
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

import os
from unittest import mock

import testscenarios

from ubuntu_sanity_tests import (
    config,
    command_line,
    selftests
)


class CommandLineTestCase(selftests.ConfigBaseTestCase):

    def setUp(self):
        super().setUp()
        self.config_stack = config.UbuntuSanityTestsStack()

    def test_setup_network_with_serial(self):
        serial = 'test_serial'
        self.set_config_values(device_serial=serial)

        command = command_line._build_setup_network_command(self.config_stack)

        self.assertEqual(['phablet-network', '-s', 'test_serial'], command)

    def test_setup_network_without_serial(self):
        self.set_config_values(device_serial='')

        command = command_line._build_setup_network_command(self.config_stack)

        self.assertEqual(['phablet-network'], command)

    def test_enable_introspection_with_serial(self):
        serial = 'test_serial'
        self.set_config_values(device_serial=serial)

        command = command_line._build_enable_autopilot_introspection_command(
            self.config_stack)

        self.assertEqual(['phablet-config', '-s', 'test_serial'], command[:3])

    def test_enable_introspection_without_serial(self):
        self.set_config_values(device_serial='')

        command = command_line._build_enable_autopilot_introspection_command(
            self.config_stack)

        self.assertNotIn('-s', command)

    def test_adt_run_must_be_called_with_short_timeout(self):
        self.set_config_values()

        command = command_line._build_adt_run_test_command(self.config_stack)
        command = ' '.join(command)

        run_arguments, _ = command.split('---')
        self.assertIn('--timeout-short=600', run_arguments)

    def test_ppa_setup_commands_set_image_readwrite(self):
        setup_ppa_commands = command_line._get_ppa_setup_command()
        self.assertIn(
            'mount -o remount,rw /',
            setup_ppa_commands
        )

    def test_ppa_setup_commands_set_image_back_to_readonly(self):
        setup_ppa_commands = command_line._get_ppa_setup_command()
        self.assertIn(
            'sync; sleep 2; mount -o remount,ro /',
            setup_ppa_commands
        )

    def test_ppa_setup_commands_only_update_ppa_details(self):
        setup_ppa_commands = command_line._get_ppa_setup_command()
        aptget_command = str('apt-get --no-list-cleanup update -o'
                             ' Dir::Etc::SourceList=/dev/null')
        self.assertIn(aptget_command, setup_ppa_commands)

    def test_ppa_setup_commands_used_in_adt_run(self):
        self.set_config_values()
        command = command_line._build_adt_run_test_command(self.config_stack)
        command = ' '.join(command)

        run_arguments, _ = command.split('---')
        self.assertIn(
            'apt-add-repository -y ppa:canonical-platform-qa/selenium',
            run_arguments
        )

    def test_adt_run_with_serial(self):
        serial = 'test_serial'
        self.set_config_values(device_serial=serial)

        command = command_line._build_adt_run_test_command(self.config_stack)
        command = ' '.join(command)

        _, adb_arguments = command.split('ssh -s adb --')
        self.assertIn('-s test_serial', adb_arguments)

    def test_adt_run_without_serial(self):
        self.set_config_values(device_serial='')

        command = command_line._build_adt_run_test_command(self.config_stack)
        command = ' '.join(command)

        _, adb_arguments = command.split('ssh -s adb --')
        self.assertNotIn('-s test_serial', adb_arguments)

    def test_adt_run_without_apt_update_must_be_called_without_setup_command(
            self):
        self.set_config_values()

        command = command_line._build_adt_run_test_command(
            self.config_stack, update_apt=False)
        command = ' '.join(command)

        run_arguments, _ = command.split('---')
        self.assertNotIn('--setup-commands ro-apt-update', run_arguments)

    def test_adt_run_with_apt_update_must_be_called_with_setup_command(self):
        self.set_config_values()

        command = command_line._build_adt_run_test_command(
            self.config_stack, update_apt=True)
        command = ' '.join(command)

        run_arguments, _ = command.split('---')
        self.assertIn('--setup-commands ro-apt-update', run_arguments)

    def test_get_config_stack_without_tests_to_run(self):
        self.set_config_values(device_serial='')

        config_stack = command_line._get_config_stack(
            'ubuntu-sanity-tests.conf', tests_to_run=[])

        self.assertEqual('', config_stack.get('tests_to_run'))

    def test_get_config_stack_with_tests_to_run(self):
        self.set_config_values(device_serial='')

        config_stack = command_line._get_config_stack(
            'ubuntu-sanity-tests.conf',
            tests_to_run=['test1', 'test2', 'test3'])

        self.assertEqual(
            'test1 test2 test3', config_stack.get('tests_to_run'))


class RunSanityTestsCommandsTestCase(selftests.ConfigBaseTestCase,
                                     testscenarios.TestWithScenarios):

    scenarios = [
        ('without network setup', {
            'keyword_arguments': {'update_apt': False,
                                  'network': False},
            'expected_commands': ['phablet-config',
                                  'adt-run'],
        }),
        ('with network setup', {
            'keyword_arguments': {'update_apt': False,
                                  'network': True},
            'expected_commands': ['phablet-network',
                                  'phablet-config',
                                  'adt-run'],
        }),
    ]

    def setUp(self):
        super().setUp()
        self.config_stack = config.UbuntuSanityTestsStack()

    @mock.patch('ubuntu_sanity_tests.command_line.generate_user_report')
    def test_run_sanity_tests_commands(self, mock_generate_user_report):
        def get_command_from_call(call):
            """Return the name of the command, without the arguments."""
            return call[0][0][0]

        self.set_config_values()

        # Do not execute the subprocesses, just record their calls.
        with mock.patch('subprocess.Popen') as mock_popen:
            enter = mock_popen().__enter__()
            # Make sure that check_output and call return with no errors.
            enter.communicate = mock.Mock(return_value=(0, 'dummy'))
            enter.poll = mock.Mock(return_value=0)
            enter.wait = mock.Mock(return_value=0)

            conf_name = os.path.basename(self.config_stack.store.path)
            command_line.run_sanity_tests(conf_name, [],
                                          **self.keyword_arguments)

        # The first call doesn't run any command.
        calls = mock_popen.call_args_list[1:]
        commands = [get_command_from_call(call) for call in calls]
        self.assertEqual(self.expected_commands, commands)


class MainPassingCommandLineArgumentsTestCase(testscenarios.TestWithScenarios):

    DEFAULT_CONFIG_FILE = 'ubuntu-sanity-tests.conf'
    RUN_ALL_TESTS = []
    DO_APT_UPDATE = True
    DONT_APT_UPDATE = False
    DO_NETWORK_SETUP = True
    DONT_NETWORK_SETUP = False

    scenarios = [
        ('no arguments', {
            'command_line_arguments': [],
            'parsed_arguments': (
                DEFAULT_CONFIG_FILE,
                RUN_ALL_TESTS,
                DONT_APT_UPDATE,
                DONT_NETWORK_SETUP)
        }),
        ('alternative config', {
            'command_line_arguments': ['--config=alternative.conf'],
            'parsed_arguments': (
                'alternative.conf',
                RUN_ALL_TESTS,
                DONT_APT_UPDATE,
                DONT_NETWORK_SETUP)
        }),
        ('with apt-get update', {
            'command_line_arguments': ['--update-apt'],
            'parsed_arguments': (
                DEFAULT_CONFIG_FILE,
                RUN_ALL_TESTS,
                DO_APT_UPDATE,
                DONT_NETWORK_SETUP)
        }),
        ('one test suite', {
            'command_line_arguments': ['test_suite'],
            'parsed_arguments': (
                DEFAULT_CONFIG_FILE,
                ['test_suite'],
                DONT_APT_UPDATE,
                DONT_NETWORK_SETUP)
        }),
        ('multiple test suites', {
            'command_line_arguments': [
                'test_suite1', 'test_suite2', 'test_suite3'
            ],
            'parsed_arguments': (
                DEFAULT_CONFIG_FILE,
                ['test_suite1', 'test_suite2', 'test_suite3'],
                DONT_APT_UPDATE, DONT_NETWORK_SETUP
            )
        }),
        ('with network setup', {
            'command_line_arguments': ['--network'],
            'parsed_arguments': (
                DEFAULT_CONFIG_FILE, [], DONT_APT_UPDATE, DO_NETWORK_SETUP)
        }),
    ]

    @mock.patch('ubuntu_sanity_tests.command_line.run_sanity_tests')
    def test_main_must_pass_command_line_arguments_to_run_sanity_tests(
            self, mock_run_sanity_tests):
        command_line.main(self.command_line_arguments)

        mock_run_sanity_tests.assert_called_once_with(
            *self.parsed_arguments)
