# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-

#
# Ubuntu Sanity Tests
# Copyright (C) 2014, 2015 Canonical
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#


# Open webbrowser-app, browse to web page, navigate forward and back
# ===================================================
# Open the browser ensure you can get to a webpage
# Steps
# --------
# 1. Move to the apps scope
# * Arrive at the apps scope
# 2. Open the browser
# * Browser open on the default start.ubuntu.com page
# 3. In the location bar type "ubuntu.com" and press enter
# * Browser open the main Ubuntu.com page
# 4. In the location bar type "planet.ubuntu.com"
# * Browers view changes to planet.ubuntu.com
# 5. Tap the back button
# * The browser return to ubuntu.com
# 6. Tap the forward button
# * The browser returns to planet.ubuntu.com

import logging
import os
import shutil
import subprocess

from autopilot import exceptions, platform
from collections import namedtuple

from ubuntu_sanity_tests.helpers import autopilot as autopilot_helpers
from ubuntu_sanity_tests.tests import base

logger = logging.getLogger(__name__)

# Testing vivid on the device but the host doesn't need to be vivid (or have
# the up-to-date helpers.)
try:
    from webbrowser_app.emulators.browser import Webbrowser
except ImportError:
    if platform.model() == 'Desktop':
        logger.info('Ignoring webrowser import error due to running on host.')
    else:
        raise


PageUrls = namedtuple('PageUrls', ['title_url', 'actual_url'])
logger = logging.getLogger(__name__)


class WebbrowserSanityTest(base.BaseUbuntuSanityTestCase):

    def setUp(self):
        super().setUp()
        self._wipe_browser_history()

    def _wipe_browser_history(self):
        """Ensure the browser starts with default settings.

        Need to make sure we start off with a clean slate. Remove the browser
        session file so we start with the default page and put it back after
        the test is done.

        """
        json_filepath = os.path.expanduser(
            '~/.local/share/webbrowser-app/session.json'
        )
        if os.path.exists(json_filepath):
            json_backup_filepath = '{}.sanity-backup'.format(json_filepath)
            logger.info(
                'Moving session file from {} to {} for the duration of '
                'this test.'.format(
                    json_filepath, json_backup_filepath
                )
            )
            shutil.move(json_filepath, json_backup_filepath)
            self.addCleanup(shutil.move, json_backup_filepath, json_filepath)

    def test_014_webbrowser(self):
        unity_proxy = self.launch_unity()
        unity_proxy.unlock()

        dash = self.get_dash()
        apps_scope = dash.open_scope('clickscope')

        apps_scope.click_scope_item('predefined', 'Browser')
        self.addCleanup(self._ensure_browser_closed)

        start_page = PageUrls(
            title_url='start.ubuntu.com',
            actual_url='http://start.ubuntu.com/current/Google/?sourceid=hp'
        )
        second_page = PageUrls(
            title_url='ubuntu.com',
            actual_url='http://www.ubuntu.com/'
        )
        third_page = PageUrls(
            title_url='planet.ubuntu.com',
            actual_url='http://planet.ubuntu.com/'
        )

        browser = autopilot_helpers.get_proxy_object_for_existing_process(
            'webbrowser-app', Webbrowser)

        # As per this bug where we're unable to launch the browser from the
        # scope after a unity8 reboot.
        # https://bugs.launchpad.net/ubuntu/+source/unity8/+bug/1401361
        self._ensure_browser_actually_loaded(unity_proxy)

        # Need to make sure that we're actually loaded initally.
        self.assert_expected_url_loaded(browser, start_page)

        browser.main_window.go_to_url(second_page.title_url)
        self.assert_expected_url_loaded(browser, second_page)

        browser.main_window.go_to_url(third_page.title_url)
        self.assert_expected_url_loaded(browser, third_page)

        browser.main_window.go_back()
        self.assert_expected_url_loaded(browser, second_page)

        browser.main_window.go_forward()
        self.assert_expected_url_loaded(browser, third_page)

    def assert_expected_url_loaded(self, browser, page_details):
        """Assert that the expected url isn't loaded.

        Ensures that the browser has loaded the url provided in
        ``page_details``.

        :param browser: Browser proxy object with which to check the url and
          loaded status of.
        :param page_details: A PageUrls named tuple object containing the title
          and actual url to check.
        :raises AssertError: If either of the title or actual urls defined in
          ``page_details`` fail to load.

        """
        browser.main_window.wait_until_page_loaded(
            page_details.actual_url
        )
        self.assertEqual(
            browser.main_window.address_bar.text,
            page_details.title_url
        )

    def _ensure_browser_actually_loaded(self, unity_proxy):
        """Fail the test if the browser app doesn't get past loading.

        Due to this bug[1] after a reboot or 2 of unity8 the browser is no
        longer launchable from the apps scope.
        Check for when this happens and if it does, fail the test with a nice
        message.

        [1] https://bugs.launchpad.net/ubuntu/+source/unity8/+bug/1401361

        """
        try:
            spinner = unity_proxy.select_single('ActivityIndicatorStyle')
            spinner.wait_until_destroyed()
        except exceptions.StateNotFoundError:
            # This is fine, it's already loaded
            pass
        except RuntimeError:
            self.fail("""Test failed due to browser not loading.
            Please see this bug for further information:
            https://bugs.launchpad.net/ubuntu/+source/unity8/+bug/1401361
            """)

    def _ensure_browser_closed(self):
        """Stop the app before unity8 is restarted so it's not orphaned.

        Bug I came across (this should solve that)
        https://bugs.launchpad.net/ubuntu/+source/unity8/+bug/1401361

        Reference bug:
        https://bugs.launchpad.net/ubuntu/+source/unity8/+bug/1326513

        """
        try:
            subprocess.call(['ubuntu-app-stop', 'webbrowser-app'])
        except:
            pass
